import bpy
import os
from .custom_icon import bl_light_presets_icons , bl_light_icons , bl_all_icons
import unicodedata


def get_bl_light_presets(self, context):
    items = []
    obj = context.active_object
    if obj and obj.type == 'LIGHT' and obj.data.node_tree:
        node_tree = obj.data.node_tree
        for i, node in enumerate(node_tree.nodes):
            if node.type == 'REROUTE' and node.name.startswith("Presets_"):
                item_label = node.label if node.label else node.name

                icon_name = node.name if node.name in bl_light_presets_icons else 'none'

                icon_id = bl_light_presets_icons[icon_name].icon_id
                
                normalized_name = unicodedata.normalize('NFKD', node.name)
                item = (normalized_name, item_label, "", icon_id, i)
                items.append(item)
    return items

current_file_path = os.path.dirname(__file__)
bl_light_icons_assets_directory = os.path.join(current_file_path, '..', 'assets', 'img','light')

def get_bl_light(self, context):
    items = []
    item_icon_name = []  
    
    for filename in os.listdir(bl_light_icons_assets_directory):
        if filename.lower().endswith(('.png', '.jpg')):
            
            icon_name = os.path.splitext(filename)[0]
            item_icon_name.append(icon_name)  

    for i, iin in enumerate(item_icon_name):
        item_label = iin

        icon_name = iin

        icon_id = bl_light_icons[icon_name].icon_id
        
        normalized_name = unicodedata.normalize('NFKD', iin)
        item = (normalized_name, item_label, "", icon_id, i)
        items.append(item)

    return items


bl_all_icons_assets_directory = os.path.join(current_file_path, '..', 'assets', 'img','All')

def get_all_presets(self, context):
    items = []
    item_icon_name = []  
    
    for filename in os.listdir(bl_all_icons_assets_directory):
        if filename.lower().endswith(('.png', '.jpg')):
            
            icon_name = os.path.splitext(filename)[0]
            item_icon_name.append(icon_name)  

    for i, iin in enumerate(item_icon_name):
        item_label = iin

        icon_name = iin

        icon_id = bl_all_icons[icon_name].icon_id
        
        normalized_name = unicodedata.normalize('NFKD', iin)
        item = (normalized_name, item_label, "", icon_id, i)
        items.append(item)

    return items

class NodeOperations:
    @staticmethod
    def find_node_return(node_tree, name):
        
        for node in node_tree.nodes:
            if node.type == 'REROUTE' and node.name == name:
                return node.name
        return None

    @staticmethod
    def connect_node(node_tree, node_a_name, node_b_name):
        
        node_a = node_tree.nodes.get(node_a_name)
        node_b = node_tree.nodes.get(node_b_name)
        if node_a and node_b:
            node_tree.links.new(node_a.outputs[0], node_b.inputs[0])

    @staticmethod
    def find_linked_nodes(node_tree, node_name):
        
        target_node = node_tree.nodes.get(node_name)
        if not target_node:
            return ""
        for input in target_node.inputs:
            for link in input.links:
                linked_node = link.from_node
                return linked_node.name
        return ""

    @staticmethod
    def change_active_light_presets_node_name(node_tree, node_name, endswith_name):

        for node in node_tree.nodes:
            if node.name.endswith(endswith_name):
                node.name = node.name.replace(endswith_name, "")
                

        
        linked_node_name = NodeOperations.find_linked_nodes(node_tree, node_name)
        if linked_node_name and not linked_node_name.endswith(endswith_name):
            node = node_tree.nodes.get(linked_node_name)
            if node:
                node.name = linked_node_name + endswith_name

    @staticmethod
    def set_light_value(node_tree, node_name, active_light):
        
        reroute_list = []
        for node in node_tree.nodes:
            if node.type == 'REROUTE' and node.name.startswith("V_"+node_name):
                reroute_list.append(node.name)
        
        for reroute in reroute_list:
            if reroute.startswith("V_"+node_name+"_shadow_soft_size="):
                shadow_soft_size = float(reroute.replace("V_"+node_name+"_shadow_soft_size=", ""))
                active_light.data.shadow_soft_size = shadow_soft_size
                



class SetLightPresetsOperator(bpy.types.Operator):
    bl_idname = "object.set_light_presets_operator"
    bl_label = "Set Light Presets"
    bl_description = "Set Light Presets"
    bl_options = {"REGISTER", "UNDO"}

    def execute(self, context):
        active_light = context.active_object
        if active_light.type == 'LIGHT':
            active_light_node_tree = bpy.data.lights[active_light.data.name].node_tree
            active_light_obj = active_light
            selected_preset = context.object.LightPresetsList.options
            
            NodeOperations.connect_node(active_light_node_tree, selected_preset, "Out")
            NodeOperations.change_active_light_presets_node_name(active_light_node_tree, selected_preset, "##")
            NodeOperations.set_light_value(active_light_node_tree, selected_preset, active_light_obj)
        return {'FINISHED'}

def update_bl_light_presets(self, context):
    bpy.ops.object.set_light_presets_operator()

def update_bl_light(self, context):


    return None


class LightPresetsList(bpy.types.PropertyGroup):
    options: bpy.props.EnumProperty(
        name="Light Presets",
        items=get_bl_light_presets,  
        update=update_bl_light_presets  
    ) 

class LightList(bpy.types.PropertyGroup):
    options: bpy.props.EnumProperty(
        name="Light",
        items=get_bl_light,
        update=update_bl_light
    ) 

class AllPresets(bpy.types.PropertyGroup):
    options: bpy.props.EnumProperty(
        name="All Presets",
        items=get_all_presets
    ) 



def register():

    bpy.types.Object.LightPresetsList = bpy.props.PointerProperty(type=LightPresetsList)
    bpy.types.Scene.LightList = bpy.props.PointerProperty(type=LightList)
    bpy.types.Scene.AllPresets = bpy.props.PointerProperty(type=AllPresets)


def unregister():

    del bpy.types.Object.LightPresetsList
    del bpy.types.Scene.LightList
    del bpy.types.Scene.AllPresets


if __name__ == "__main__":
    register()